@@ -36,6 +36,7 @@ import os
 import sys
 
 import requests
+import six
 
 # Byte Order Mark. https://en.wikipedia.org/wiki/Byte_order_mark
 _BOM = u'\ufeff'
@@ -76,16 +77,18 @@ class Client(object):
                 return token.value
 
         auth_request = requests.post(
-            'https://entreprise.pole-emploi.fr/connexion/oauth2/access_token',
+            'https://entreprise.pole-emploi.fr/connexion/oauth2/access_token?realm=%2Fpartenaire',
             data={
-                'realm': '/partenaire',
                 'grant_type': 'client_credentials',
                 'client_id': self._client_id,
                 'client_secret': self._client_secret,
                 'scope': 'application_%s %s' % (self._client_id, scope),
             })
-        if auth_request.status_code != 200:
-            raise ValueError('Client ID or secret invalid')
+        try:
+            auth_request.raise_for_status()
+        except Exception as error:
+            raise ValueError('Autentication failed: {}'.format(error))
+
         response = auth_request.json()
         token = response.get('access_token')
         expires_in = datetime.timedelta(seconds=response.get('expires_in', 600))
@@ -99,9 +102,7 @@ class Client(object):
         req = requests.get(
             self.api_url + '/infotravail/v1' + action, params=params,
             headers={'Authorization': 'Bearer %s' % self.access_token(scope)})
-        if req.status_code != 200:
-            raise EnvironmentError(
-                'HTTP error %d for: \n%s' % (req.status_code, req.url))
+        req.raise_for_status()
         response = req.json()
         if not response.get('success'):
             return None
@@ -183,9 +184,7 @@ class Client(object):
         req = requests.get(
             self.api_url + '/labonneboite/v1/company/', params=params,
             headers={'Authorization': 'Bearer %s' % self.access_token(scope)})
-        if req.status_code != 200:
-            raise EnvironmentError(
-                'HTTP error %d for: \n%s' % (req.status_code, req.url))
+        req.raise_for_status()
         response = req.json()
         companies = response.get('companies')
         if companies:
@@ -213,12 +212,37 @@ class Client(object):
             self.api_url + '/retouralemploisuiteformation/v1/rank',
             params={'formacode': formacode, 'codeinseeville': city_id},
             headers={'Authorization': 'Bearer %s' % self.access_token(scope)})
-        if req.status_code != 200:
-            raise EnvironmentError(
-                'HTTP error %d for: \n%s' % (req.status_code, req.url))
+        req.raise_for_status()
         response = req.json()
         return response[0]
 
+    def get_match_via_soft_skills(self, rome):
+        """Get the soft skills for a specific job group sorted by significance.
+
+        See documentation at:
+            https://www.emploi-store-dev.fr/portail-developpeur-cms/home/catalogue-des-api/documentation-des-api/api-matchviasoftskills-v1.html
+
+        Args:
+            rome: unique ID for the job group. See
+                https://www.pole-emploi.fr/candidat/le-code-rome-et-les-fiches-metiers-@/article.jspz?id=60702
+
+        Yields:
+            a dict per skill, see
+            https://www.emploi-store-dev.fr/portail-developpeur-cms/home/catalogue-des-api/documentation-des-api/api-matchviasoftskills-v1.html
+            for details of the fields.
+        """
+        scope = 'api_matchviasoftskillsv1'
+        req = requests.post(
+            self.api_url + '/matchviasoftskills/v1/professions/job_skills',
+            params={'code': rome},
+            headers={'Authorization': 'Bearer %s' % self.access_token(scope)})
+        req.raise_for_status()
+        response = req.json()
+        skills = response.get('skills')
+        if skills:
+            for skill in skills:
+                yield skills[skill]
+
     def list_emploistore_services(self):
         """List all the user-facing services proposed by the Emploi Store.
 
@@ -272,7 +296,7 @@ class Client(object):
 
         Returns:
             a list of dicts, see
-            https://www.emploi-store-dev.fr/portail-developpeur-cms/home/catalogue-des-api/documentation-des-api/api-evenements-pole-emploi-v1/rechercher-les-salons-en-ligne.html
+            https://www.emploi-store-dev.fr/portail-developpeur-cms/home/catalogue-des-api/documentation-des-api/api/api-evenements-pole-emploi-v1/rechercher-les-salons-en-ligne.html
             for details of the fields.
         """
         scope = 'api_evenementsv1 evenements'
@@ -285,6 +309,24 @@ class Client(object):
         req.raise_for_status()
         return req.json()
 
+    def list_physical_events(self):
+        """List physical events.
+
+        Returns:
+            a list of dicts, see
+            https://www.emploi-store-dev.fr/portail-developpeur-cms/home/catalogue-des-api/documentation-des-api/api/api-evenements-pole-emploi-v1/rechercher-les-evenements-physiq.html
+            for details of the fields.
+        """
+        scope = 'api_evenementsv1 evenements'
+        req = requests.get(
+            self.api_url + '/evenements/v1/evenementsphysiques',
+            headers={
+                'Accept': 'application/json',
+                'Authorization': 'Bearer %s' % self.access_token(scope),
+            })
+        req.raise_for_status()
+        return req.json()
+
 
 class Package(object):
     """A package of resources available.
@@ -372,13 +414,13 @@ class Resource(object):
                 modify the records or just keep track of progress.
         """
         records = self.records(batch_size=batch_size, filters=filters)
-        need_utf8_encode = sys.version_info < (3, 0)
+        need_utf8_encode = six.PY2
         if not fieldnames:
             first = records.peek_first()
             keys = set(_strip_bom(k) for k in first.keys())
             fieldnames = sorted(keys - set(['_id']))
         if need_utf8_encode:
-            fieldnames = [f.encode('utf-8') for f in fieldnames]
+            fieldnames = [six.ensure_str(f, encoding='utf-8') for f in fieldnames]
         with open(file_name, 'wt') as csvfile:
             csv_writer = csv.DictWriter(
                 csvfile, fieldnames, extrasaction='ignore')
@@ -387,7 +429,8 @@ class Resource(object):
                 record = {_strip_bom(k): v for k, v in record.items()}
                 if need_utf8_encode:
                     record = {
-                        k.encode('utf-8'): unicode(v).encode('utf-8')
+                        six.ensure_str(k, encoding='utf-8'):
+                        six.ensure_str(six.text_type(v), encoding='utf-8')
                         for k, v in record.items()}
                 csv_writer.writerow(record)
 
@@ -417,6 +460,7 @@ class _ResourceIterator(object):
 
     # For Python 2 compatibility.
     def next(self):
+        """Get next resource."""
         return next(self._generator)
 
     def _create_generator(self):
